@@ -1,8 +1,10 @@
 # -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """
-Framework and base classes for coordinate frames/"low-level" coordinate classes.
+Framework and base classes for coordinate frames/"low-level" coordinate
+classes.
 """
+
 from __future__ import (absolute_import, unicode_literals, division,
                         print_function)
 
@@ -13,50 +15,60 @@ from copy import deepcopy
 from collections import namedtuple
 
 # Dependencies
+import numpy as np
 
 # Project
 from ..utils.compat.misc import override__dir__
 from ..extern import six
-from ..utils.compat.odict import OrderedDict
-from ..utils.exceptions import AstropyDeprecationWarning
+from ..utils.exceptions import AstropyDeprecationWarning, AstropyWarning
 from .. import units as u
 from ..utils import OrderedDict
 from .transformations import TransformGraph
 from .representation import (BaseRepresentation, CartesianRepresentation,
-                             SphericalRepresentation, UnitSphericalRepresentation,
+                             SphericalRepresentation,
+                             UnitSphericalRepresentation,
                              REPRESENTATION_CLASSES)
+from .earth import EarthLocation
+
+
+__all__ = ['BaseCoordinateFrame', 'frame_transform_graph', 'GenericFrame',
+           'FrameAttribute', 'TimeFrameAttribute', 'QuantityFrameAttribute',
+           'EarthLocationAttribute', 'RepresentationMapping']
 
-__all__ = ['BaseCoordinateFrame', 'frame_transform_graph', 'GenericFrame', 'FrameAttribute',
-           'TimeFrameAttribute', 'RepresentationMapping']
 
 # the graph used for all transformations between frames
 frame_transform_graph = TransformGraph()
 
 
 def _get_repr_cls(value):
-    """Return a valid representation class from ``value`` or raise exception."""
+    """
+    Return a valid representation class from ``value`` or raise exception.
+    """
+
     if value in REPRESENTATION_CLASSES:
         value = REPRESENTATION_CLASSES[value]
     try:
-        assert issubclass(value, BaseRepresentation)  # value might not be a class, so use try
-    except:
-        raise ValueError('representation is {0!r} but must be a BaseRepresentation class '
-                         ' or one of the string aliases {1}'
-                         .format(value, list(REPRESENTATION_CLASSES)))
+        # value might not be a class, so use try
+        assert issubclass(value, BaseRepresentation)
+    except (TypeError, AssertionError):
+        raise ValueError(
+            'Representation is {0!r} but must be a BaseRepresentation class '
+            'or one of the string aliases {1}'.format(
+                value, list(REPRESENTATION_CLASSES)))
     return value
 
 
 class FrameMeta(type):
-    def __new__(cls, name, parents, clsdct):
-        if 'default_representation' in clsdct:
-            def_repr = clsdct.pop('default_representation')
-            found_def_repr = True
+    def __new__(mcls, name, bases, members):
+        if 'default_representation' in members:
+            default_repr = members.pop('default_representation')
+            found_default_repr = True
         else:
-            def_repr = None
-            found_def_repr = False
+            default_repr = None
+            found_default_repr = False
 
-        if 'frame_specific_representation_info' in clsdct:
-            repr_info = clsdct.pop('frame_specific_representation_info')
+        if 'frame_specific_representation_info' in members:
+            repr_info = members.pop('frame_specific_representation_info')
             found_repr_info = True
         else:
             repr_info = None
@@ -64,46 +76,51 @@ class FrameMeta(type):
 
         # somewhat hacky, but this is the best way to get the MRO according to
         # https://mail.python.org/pipermail/python-list/2002-December/167861.html
-        mro = super(FrameMeta, cls).__new__(cls, name, parents, clsdct).__mro__
-        parent_clsdcts = [c.__dict__ for c in mro]
+        tmp_cls = super(FrameMeta, mcls).__new__(mcls, name, bases, members)
 
-        #now look through the whole MRO for the class attributes, raw
-        # for frame_attr_names, and leading underscore for others
-        for clsdcti in parent_clsdcts:
-            if not found_def_repr and '_default_representation' in clsdcti:
-                def_repr = clsdcti['_default_representation']
-                found_def_repr = True
-            if not found_repr_info and '_frame_specific_representation_info' in clsdcti:
-                repr_info = clsdcti['_frame_specific_representation_info']
+        # now look through the whole MRO for the class attributes, raw for
+        # frame_attr_names, and leading underscore for others
+        for m in (c.__dict__ for c in tmp_cls.__mro__):
+            if not found_default_repr and '_default_representation' in m:
+                default_repr = m['_default_representation']
+                found_default_repr = True
+            if (not found_repr_info and
+                    '_frame_specific_representation_info' in m):
+                repr_info = m['_frame_specific_representation_info']
                 found_repr_info = True
 
-            if found_def_repr and found_repr_info:
+            if found_default_repr and found_repr_info:
                 break
         else:
-            raise ValueError('Could not find all expected BaseCoordinateFrame '
-                             'class attributes.  Are you mis-using FrameMeta?')
+            raise ValueError(
+                'Could not find all expected BaseCoordinateFrame class '
+                'attributes.  Are you mis-using FrameMeta?')
 
         # Make read-only properties for the frame class attributes that should
         # be read-only to make them immutable after creation.
         # We copy attributes instead of linking to make sure there's no
         # accidental cross-talk between classes
-        clsdct['_default_representation'] = def_repr
-        clsdct['default_representation'] = FrameMeta.readonly_prop_factory('default_representation')
-
-        clsdct['_frame_specific_representation_info'] = deepcopy(repr_info)
-        clsdct['frame_specific_representation_info'] = FrameMeta.readonly_prop_factory('frame_specific_representation_info')
+        mcls.readonly_prop_factory(members, 'default_representation',
+                                   default_repr)
+        mcls.readonly_prop_factory(members,
+                                   'frame_specific_representation_info',
+                                   deepcopy(repr_info))
 
         # now set the frame name as lower-case class name, if it isn't explicit
-        if 'name' not in clsdct:
-            clsdct['name'] = name.lower()
+        if 'name' not in members:
+            members['name'] = name.lower()
 
-        return super(FrameMeta, cls).__new__(cls, name, parents, clsdct)
+        return super(FrameMeta, mcls).__new__(mcls, name, bases, members)
 
     @staticmethod
-    def readonly_prop_factory(attrnm):
+    def readonly_prop_factory(members, attr, value):
+        private_attr = '_' + attr
+
         def getter(self):
-            return getattr(self, '_' + attrnm)
-        return property(getter)
+            return getattr(self, private_attr)
+
+        members[private_attr] = value
+        members[attr] = property(getter)
 
 
 class FrameAttribute(object):
@@ -118,7 +135,8 @@ class FrameAttribute(object):
 
       class FK4(BaseCoordinateFrame):
           equinox = TimeFrameAttribute(default=_EQUINOX_B1950)
-          obstime = TimeFrameAttribute(default=None, secondary_attribute='equinox')
+          obstime = TimeFrameAttribute(default=None,
+                                       secondary_attribute='equinox')
 
     This means that ``equinox`` and ``obstime`` are available to be set as
     keyword arguments when creating an ``FK4`` class instance and are then
@@ -136,13 +154,8 @@ class FrameAttribute(object):
     default : object
         Default value for the attribute if not provided
     secondary_attribute : str
-        Name of a secondary instance attribute which supplies the value
-        if ``default is None`` and no value was supplied during initialization.
-
-    Returns
-    -------
-    frame_attr : descriptor
-        A new data descriptor to hold a frame attribute
+        Name of a secondary instance attribute which supplies the value if
+        ``default is None`` and no value was supplied during initialization.
     """
 
     _nextid = 1
@@ -168,10 +181,10 @@ class FrameAttribute(object):
         as needed.  The method should catch any internal exceptions and raise
         ValueError with an informative message.
 
-        The method returns the validated input along with a boolean that indicates
-        whether the input value was actually converted.  If the input value was
-        already the correct type then the ``converted`` return value should be
-        ``False``.
+        The method returns the validated input along with a boolean that
+        indicates whether the input value was actually converted.  If the input
+        value was already the correct type then the ``converted`` return value
+        should be ``False``.
 
         Parameters
         ----------
@@ -195,8 +208,9 @@ class FrameAttribute(object):
 
     def __get__(self, instance, frame_cls=None):
         if not hasattr(self, 'name'):
-            # Find attribute name of self by finding this object in the frame class
-            # which is requesting this attribute or any of its superclasses.
+            # Find attribute name of self by finding this object in the frame
+            # class which is requesting this attribute or any of its
+            # superclasses.
             for mro_cls in frame_cls.__mro__:
                 for name, val in mro_cls.__dict__.items():
                     if val is self:
@@ -205,10 +219,12 @@ class FrameAttribute(object):
                 if hasattr(self, 'name'):  # Can't nicely break out of two loops
                     break
             else:
-                # Cannot think of a way to actually raise this exception.  This instance
-                # containing this code must be in the class dict in order to get excecuted
-                # by attribute access.  But leave this here just in case...
-                raise AttributeError('Unexpected inability to locate descriptor')
+                # Cannot think of a way to actually raise this exception.  This
+                # instance containing this code must be in the class dict in
+                # order to get excecuted by attribute access.  But leave this
+                # here just in case...
+                raise AttributeError(
+                        'Unexpected inability to locate descriptor')
 
         out = None
 
@@ -241,18 +257,14 @@ class TimeFrameAttribute(FrameAttribute):
     default : object
         Default value for the attribute if not provided
     secondary_attribute : str
-        Name of a secondary instance attribute which supplies the value
-        if ``default is None`` and no value was supplied during initialization.
-
-    Returns
-    -------
-    frame_attr : descriptor
-        A new data descriptor to hold a frame attribute
+        Name of a secondary instance attribute which supplies the value if
+        ``default is None`` and no value was supplied during initialization.
     """
+
     def convert_input(self, value):
         """
-        Convert input value to a Time object and validate by running through the
-        Time constructor.  Also check that the input was a scalar.
+        Convert input value to a Time object and validate by running through
+        the Time constructor.  Also check that the input was a scalar.
 
         Parameters
         ----------
@@ -270,6 +282,7 @@ class TimeFrameAttribute(FrameAttribute):
         ValueError
             If the input is not valid for this attribute.
         """
+
         from ..time import Time
 
         if value is None:
@@ -282,32 +295,150 @@ class TimeFrameAttribute(FrameAttribute):
             try:
                 out = Time(value)
             except Exception as err:
-                raise ValueError('Invalid time input {0}={1!r}\n{2}'
-                                 .format(self.name, value, err))
+                raise ValueError(
+                    'Invalid time input {0}={1!r}\n{2}'.format(self.name,
+                                                               value, err))
             converted = True
 
-        if not out.isscalar:
-            raise ValueError('Time input {0}={1!r} must be a single (scalar) value'
-                             .format(self.name, value))
-
         return out, converted
 
 
-class RepresentationMapping(namedtuple('RepresentationMapping',
-                            ['reprname', 'framename', 'defaultunit'])):
+class QuantityFrameAttribute(FrameAttribute):
+    """
+    A frame attribute that is a quantity with specified units and shape
+    (optionally).
+
+    Parameters
+    ----------
+    default : object
+        Default value for the attribute if not provided
+    secondary_attribute : str
+        Name of a secondary instance attribute which supplies the value if
+        ``default is None`` and no value was supplied during initialization.
+    unit : unit object or None
+        Name of a unit that the input will be converted into. If None, no
+        unit-checking or conversion is performed
+    shape : tuple or None
+        If given, specifies the shape the attribute must be
+    """
+    def __init__(self, default=None, secondary_attribute='', unit=None, shape=None):
+        super(QuantityFrameAttribute, self).__init__(default, secondary_attribute)
+        self.unit = unit
+        self.shape = shape
+
+    def convert_input(self, value):
+        """
+        Checks that the input is a Quantity with the necessary units (or the
+        special value ``0``).
+
+        Parameters
+        ----------
+        value : object
+            Input value to be converted.
+
+        Returns
+        -------
+        out, converted : correctly-typed object, boolean
+            Tuple consisting of the correctly-typed object and a boolean which
+            indicates if conversion was actually performed.
+
+        Raises
+        ------
+        ValueError
+            If the input is not valid for this attribute.
+        """
+        if np.all(value == 0) and self.unit is not None and self.unit is not None:
+            return u.Quantity(np.zeros(self.shape), self.unit), True
+        else:
+            converted = True
+            if not (hasattr(value, 'unit') ):
+                raise TypeError('Tried to set a QuantityFrameAttribute with '
+                                'something that does not have a unit.')
+            oldvalue = value
+            value = u.Quantity(oldvalue, copy=False).to(self.unit)
+            if self.shape is not None and value.shape != self.shape:
+                raise ValueError('The provided value has shape "{0}", but '
+                                 'should have shape "{1}"'.format(value.shape,
+                                                                  self.shape))
+            if (oldvalue.unit == value.unit and hasattr(oldvalue, 'value') and
+                np.all(oldvalue.value == value.value)):
+                converted = False
+            return value, converted
+
+
+class EarthLocationAttribute(FrameAttribute):
+    """
+    A frame attribute that can act as a `~astropy.coordinates.EarthLocation`.
+    It can be created as anything that can be transformed to the
+    `~astropy.coordinates.ITRS` frame, but always presents as an `EarthLocation`
+    when accessed after creation.
+
+    Parameters
+    ----------
+    default : object
+        Default value for the attribute if not provided
+    secondary_attribute : str
+        Name of a secondary instance attribute which supplies the value if
+        ``default is None`` and no value was supplied during initialization.
+    """
+
+    def convert_input(self, value):
+        """
+        Checks that the input is a Quantity with the necessary units (or the
+        special value ``0``).
+
+        Parameters
+        ----------
+        value : object
+            Input value to be converted.
+
+        Returns
+        -------
+        out, converted : correctly-typed object, boolean
+            Tuple consisting of the correctly-typed object and a boolean which
+            indicates if conversion was actually performed.
+
+        Raises
+        ------
+        ValueError
+            If the input is not valid for this attribute.
+        """
+        if value is None:
+            return None, False
+        elif isinstance(value, EarthLocation):
+            return value, False
+        else:
+            #we have to do the import here because of some tricky circular deps
+            from .builtin_frames import ITRS
+
+            if not hasattr(value, 'transform_to'):
+                raise ValueError('"{0}" was passed into an '
+                                 'EarthLocationAttribute, but it does not have '
+                                 '"transform_to" method'.format(value))
+            itrsobj = value.transform_to(ITRS)
+            return itrsobj.earth_location, True
+
+_RepresentationMappingBase = \
+    namedtuple('RepresentationMapping',
+               ('reprname', 'framename', 'defaultunit'))
+
+
+class RepresentationMapping(_RepresentationMappingBase):
     """
     This `~collections.namedtuple` is used with the
     ``frame_specific_representation_info`` attribute to tell frames what
     attribute names (and default units) to use for a particular representation.
     ``reprname`` and ``framename`` should be strings, while ``defaultunit`` can
-    be either an astropy unit, the string ``'recommended'`` (to use whatever the
-    representation's ``recommended_units`` is), or None (to indicate that no
-    unit mapping should be done).
+    be either an astropy unit, the string ``'recommended'`` (to use whatever
+    the representation's ``recommended_units`` is), or None (to indicate that
+    no unit mapping should be done).
     """
+
     def __new__(cls, reprname, framename, defaultunit='recommended'):
         # this trick just provides some defaults
         return super(RepresentationMapping, cls).__new__(cls, reprname,
-                                                         framename, defaultunit)
+                                                         framename,
+                                                         defaultunit)
 
 
 @six.add_metaclass(FrameMeta)
@@ -329,15 +460,15 @@ class BaseCoordinateFrame(object):
        built-in classes code for details.
 
     * `frame_specific_representation_info`
-        A dictionary mapping the name or class of a representation to a list
-        of `~astropy.coordinates.RepresentationMapping` objects that tell what
+        A dictionary mapping the name or class of a representation to a list of
+        `~astropy.coordinates.RepresentationMapping` objects that tell what
         names and default units should be used on this frame for the components
         of that representation.
-
     """
 
     default_representation = None
-    frame_specific_representation_info = {}  # specifies special names/units for representation attributes
+    # specifies special names/units for representation attributes
+    frame_specific_representation_info = {}
 
     # This __new__ provides for backward-compatibility with pre-0.4 API.
     # TODO: remove in 1.0
@@ -345,12 +476,14 @@ class BaseCoordinateFrame(object):
 
         # Only do backward-compatibility if frame is previously defined one
         frame_name = cls.__name__.lower()
-        if frame_name not in ['altaz', 'fk4', 'fk4noeterms', 'fk5', 'galactic', 'icrs']:
+        if frame_name not in ['altaz', 'fk4', 'fk4noeterms', 'fk5',
+                              'galactic', 'icrs']:
             return super(BaseCoordinateFrame, cls).__new__(cls)
 
         use_skycoord = False
 
-        if len(args) > 1 or (len(args) == 1 and not isinstance(args[0], BaseRepresentation)):
+        if (len(args) > 1 or (len(args) == 1 and
+                not isinstance(args[0], BaseRepresentation))):
             for arg in args:
                 if (not isinstance(arg, u.Quantity)
                     and not isinstance(arg, BaseRepresentation)):
@@ -358,7 +491,7 @@ class BaseCoordinateFrame(object):
                            'or other non-Quantity arguments is deprecated, and '
                            'will be removed in the next version of Astropy.  '
                            'Instead, you probably want to use the SkyCoord '
-                           'class with the "system={1}" keyword, or if you '
+                           'class with the "frame={1}" keyword, or if you '
                            'really want to use the low-level frame classes, '
                            'create it with an Angle or Quantity.')
 
@@ -369,21 +502,26 @@ class BaseCoordinateFrame(object):
                     break
 
         if 'unit' in kwargs and not use_skycoord:
-            warnings.warn("Initializing frames using the ``unit`` argument is "
-                          "now deprecated. Use SkyCoord or pass Quantity "
-                          " instances to frames instead.", AstropyDeprecationWarning)
+            warnings.warn(
+                "Initializing frames using the ``unit`` argument is "
+                "now deprecated. Use SkyCoord or pass Quantity "
+                "instances to frames instead.", AstropyDeprecationWarning)
             use_skycoord = True
 
         if not use_skycoord:
-            representation = _get_repr_cls(kwargs.get('representation',
-                                                      cls._default_representation))
-            for key in cls._get_representation_info()[representation]['names']:
+            representation = kwargs.get('representation',
+                                        cls._default_representation)
+            representation = _get_repr_cls(representation)
+
+            repr_info = cls._get_representation_info()
+
+            for key in repr_info[representation]['names']:
                 if key in kwargs:
                     if not isinstance(kwargs[key], u.Quantity):
-                        warnings.warn("Initializing frames using non-Quantity "
-                                      "arguments is now deprecated. Use "
-                                      "SkyCoord or pass Quantity instances "
-                                      "instead.", AstropyDeprecationWarning)
+                        warnings.warn(
+                            "Initializing frames using non-Quantity arguments "
+                            "is now deprecated. Use SkyCoord or pass Quantity "
+                            "instances instead.", AstropyDeprecationWarning)
                         use_skycoord = True
                         break
 
@@ -400,7 +538,8 @@ class BaseCoordinateFrame(object):
         if 'representation' in kwargs:
             self.representation = kwargs.pop('representation')
 
-        representation_data = None  # if not set below, this is a frame with no data
+        # if not set below, this is a frame with no data
+        representation_data = None
 
         for fnm, fdefault in self.get_frame_attr_names().items():
             # Read-only frame attributes are defined as FrameAttribue
@@ -424,8 +563,9 @@ class BaseCoordinateFrame(object):
                                 args[0] is None):
             representation_data = args.pop(0)
             if len(args) > 0:
-                raise TypeError('Cannot create a frame with both a '
-                                'representation and other positional arguments')
+                raise TypeError(
+                    'Cannot create a frame with both a representation and '
+                    'other positional arguments')
 
         elif self.representation:
             repr_kwargs = {}
@@ -442,18 +582,20 @@ class BaseCoordinateFrame(object):
             if repr_kwargs:
                 if repr_kwargs.get('distance', True) is None:
                     del repr_kwargs['distance']
-                if (self.representation == SphericalRepresentation and
+                if (issubclass(self.representation, SphericalRepresentation) and
                         'distance' not in repr_kwargs):
                     representation_data = UnitSphericalRepresentation(**repr_kwargs)
                 else:
                     representation_data = self.representation(**repr_kwargs)
 
         if len(args) > 0:
-            raise TypeError(self.__class__.__name__ + '.__init__ had {0} '
-                            'remaining unprocessed arguments'.format(len(args)))
+            raise TypeError(
+                '{0}.__init__ had {1} remaining unhandled arguments'.format(
+                    self.__class__.__name__, len(args)))
         if kwargs:
-            raise TypeError('Coordinate frame got unexpected keywords: ' +
-                            str(kwargs.keys()))
+            raise TypeError(
+                'Coordinate frame got unexpected keywords: {0}'.format(
+                    list(kwargs)))
 
         self._data = representation_data
 
@@ -620,8 +762,9 @@ class BaseCoordinateFrame(object):
             A new object with the same frame attributes as this one, but
             with the ``representation`` as the data.
         """
-        frattrs = dict([(nm, getattr(self, nm)) for nm in self.get_frame_attr_names()
-                        if nm not in self._attr_names_with_defaults])
+        frattrs = dict([(attr, getattr(self, attr))
+                        for attr in self.get_frame_attr_names()
+                        if attr not in self._attr_names_with_defaults])
         return self.__class__(representation, **frattrs)
 
     def represent_as(self, new_representation, in_frame_units=False):
@@ -660,15 +803,18 @@ class BaseCoordinateFrame(object):
         >>> from astropy.coordinates import SkyCoord, CartesianRepresentation
         >>> coord = SkyCoord(0*u.deg, 0*u.deg)
         >>> coord.represent_as(CartesianRepresentation)
-        <CartesianRepresentation x=1.0, y=0.0, z=0.0>
+        <CartesianRepresentation (x, y, z) [dimensionless]
+                (1.0, 0.0, 0.0)>
 
         >>> coord.representation = CartesianRepresentation
         >>> coord
-        <SkyCoord (ICRS): x=1.0, y=0.0, z=0.0>
+        <SkyCoord (ICRS): (x, y, z) [dimensionless]
+            (1.0, 0.0, 0.0)>
         """
         new_representation = _get_repr_cls(new_representation)
 
-        cached_repr = self._rep_cache.get((new_representation.__name__, in_frame_units))
+        cached_repr = self._rep_cache.get((new_representation.__name__,
+                                           in_frame_units))
         if not cached_repr:
             data = self.data.represent_as(new_representation)
 
@@ -676,7 +822,8 @@ class BaseCoordinateFrame(object):
             # set of names and units, then use that.
             new_attrs = self.representation_info.get(new_representation)
             if new_attrs and in_frame_units:
-                datakwargs = dict((comp, getattr(data, comp)) for comp in data.components)
+                datakwargs = dict((comp, getattr(data, comp))
+                                  for comp in data.components)
                 for comp, new_attr_unit in zip(data.components, new_attrs['units']):
                     if new_attr_unit:
                         datakwargs[comp] = datakwargs[comp].to(new_attr_unit)
@@ -756,12 +903,11 @@ class BaseCoordinateFrame(object):
                 coord2 = coord.transform_to(some_unknown_frame)
 
         This will work even if ``some_unknown_frame``  turns out to be the same
-        frame class as ``coord``.  This is intended for cases where the frame is
-        the same regardless of the frame attributes (e.g. ICRS), but be aware
-        that it *might* also indicate that someone forgot to define the
+        frame class as ``coord``.  This is intended for cases where the frame
+        is the same regardless of the frame attributes (e.g. ICRS), but be
+        aware that it *might* also indicate that someone forgot to define the
         transformation between two objects of the same frame class but with
         different attributes.
-
         """
 
         new_frame_cls = new_frame if inspect.isclass(new_frame) else new_frame.__class__
@@ -789,22 +935,58 @@ class BaseCoordinateFrame(object):
         Returns
         -------
         isdefault : bool
-            True if the attribute ``attrnm`` has its value by default, False if it
-            was specified at creation of this frame.
+            True if the attribute ``attrnm`` has its value by default, False if
+            it was specified at creation of this frame.
         """
         return attrnm in self._attr_names_with_defaults
 
+    def is_equivalent_frame(self, other):
+        """
+        Checks if this object is the same frame as the ``other`` object.
+
+        To be the same frame, two objects must be the same frame class and have
+        the same frame attributes.  Note that it does *not* matter what, if any,
+        data either object has.
+
+        Parameters
+        ----------
+        other : BaseCoordinateFrame
+            the other frame to check
+
+        Returns
+        -------
+        isequiv : bool
+            True if the frames are the same, False if not.
+
+        Raises
+        ------
+        TypeError
+            If ``other`` isn't a `BaseCoordinateFrame` or subclass.
+        """
+        if self.__class__ == other.__class__:
+            for frame_attr_name in self.get_frame_attr_names():
+                if getattr(self, frame_attr_name) != getattr(other, frame_attr_name):
+                    return False
+            return True
+        elif not isinstance(other, BaseCoordinateFrame):
+            raise TypeError("Tried to do is_equivalent_frame on something that "
+                            "isn't a frame")
+        else:
+            return False
+
     def __repr__(self):
         frameattrs = ', '.join([attrnm + '=' + str(getattr(self, attrnm))
                                 for attrnm in self.get_frame_attr_names()])
 
         if self.has_data:
             if self.representation:
-                if (self.representation == SphericalRepresentation and
+                if (issubclass(self.representation, SphericalRepresentation) and
                         isinstance(self.data, UnitSphericalRepresentation)):
-                    data = self.represent_as(UnitSphericalRepresentation, in_frame_units=True)
+                    data = self.represent_as(self.data.__class__,
+                                             in_frame_units=True)
                 else:
-                    data = self.represent_as(self.representation, in_frame_units=True)
+                    data = self.represent_as(self.representation,
+                                             in_frame_units=True)
 
                 data_repr = repr(data)
                 for nmpref, nmrepr in self.representation_component_names.items():
@@ -817,23 +999,23 @@ class BaseCoordinateFrame(object):
             if data_repr.startswith('<' + data.__class__.__name__):
                 # standard form from BaseRepresentation
                 if frameattrs:
-                    frameattrs = frameattrs + ', '
+                    frameattrs = ' (' + frameattrs + ')'
 
                 #remove both the leading "<" and the space after the name
                 data_repr = data_repr[(len(data.__class__.__name__) + 2):]
 
-                return '<{0} Coordinate: {1}{2}'.format(self.__class__.__name__,
+                return '<{0} Coordinate{1}: {2}'.format(self.__class__.__name__,
                                                         frameattrs, data_repr)
             else:
                 # should only happen if a representation has a non-standard
                 # __repr__ method, and we just punt to that
                 if frameattrs:
-                    frameattrs = ': ' + frameattrs + ', '
+                    frameattrs = ' (' + frameattrs + '), '
                 s = '<{0} Coordinate{1}Data:\n{2}>'
                 return s.format(self.__class__.__name__, frameattrs, data_repr)
         else:
             if frameattrs:
-                frameattrs = ': ' + frameattrs
+                frameattrs = ' (' + frameattrs + ')'
             return '<{0} Frame{1}>'.format(self.__class__.__name__, frameattrs)
 
     def __getitem__(self, view):
@@ -856,14 +1038,20 @@ class BaseCoordinateFrame(object):
 
     def __getattr__(self, attr):
         """
-        Allow access to attributes defined in self.representation_component_names.
+        Allow access to attributes defined in
+        ``self.representation_component_names``.
 
-        TODO: dynamic representation transforms (i.e. include cylindrical et al.).
+        TODO: dynamic representation transforms (i.e. include cylindrical et
+        al.).
         """
+
         # attr == '_representation' is likely from the hasattr() test in the
-        # representation property which is used for self.representation_component_names.
+        # representation property which is used for
+        # self.representation_component_names.
+        #
         # Prevent infinite recursion here.
-        if attr == '_representation' or attr not in self.representation_component_names:
+        if (attr == '_representation' or
+                attr not in self.representation_component_names):
             raise AttributeError("'{0}' object has no attribute '{1}'"
                                  .format(self.__class__.__name__, attr))
 
@@ -877,7 +1065,8 @@ class BaseCoordinateFrame(object):
             for representation_attr in self.representation_info.values():
                 repr_attr_names.extend(representation_attr['names'])
         if attr in repr_attr_names:
-            raise AttributeError('Cannot set any frame attribute {0}'.format(attr))
+            raise AttributeError(
+                'Cannot set any frame attribute {0}'.format(attr))
         else:
             super(BaseCoordinateFrame, self).__setattr__(attr, value)
 
@@ -907,7 +1096,8 @@ class BaseCoordinateFrame(object):
         from .angles import Angle
 
         self_unit_sph = self.represent_as(UnitSphericalRepresentation)
-        other_unit_sph = other.transform_to(self.__class__).represent_as(UnitSphericalRepresentation)
+        other_transformed = other.transform_to(self.__class__)
+        other_unit_sph = other_transformed.represent_as(UnitSphericalRepresentation)
 
         # Get the separation as a Quantity, convert to Angle in degrees
         sep = angular_separation(self_unit_sph.lon, self_unit_sph.lat,
@@ -934,6 +1124,7 @@ class BaseCoordinateFrame(object):
         ValueError
             If this or the other coordinate do not have distances.
         """
+
         from .distances import Distance
 
         if self.data.__class__ == UnitSphericalRepresentation:
@@ -957,8 +1148,10 @@ class BaseCoordinateFrame(object):
     @property
     def cartesian(self):
         """
-        Shorthand for a cartesian representation of the coordinates in this object.
+        Shorthand for a cartesian representation of the coordinates in this
+        object.
         """
+
         # TODO: if representations are updated to use a full transform graph,
         #       the representation aliases should not be hard-coded like this
         return self.represent_as(CartesianRepresentation, in_frame_units=True)
@@ -968,6 +1161,7 @@ class BaseCoordinateFrame(object):
         """
         Shorthand for a spherical representation of the coordinates in this object.
         """
+
         # TODO: if representations are updated to use a full transform graph,
         #       the representation aliases should not be hard-coded like this
         return self.represent_as(SphericalRepresentation, in_frame_units=True)
@@ -1008,5 +1202,3 @@ class GenericFrame(BaseCoordinateFrame):
             raise AttributeError("can't set frame attribute '{0}'".format(name))
         else:
             super(GenericFrame, self).__setattr__(name, value)
-
-
