@@ -8,6 +8,7 @@ from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
 import abc
+import functools
 
 import numpy as np
 import astropy.units as u
@@ -16,28 +17,19 @@ from .angles import Angle, Longitude, Latitude
 from .distances import Distance
 from ..extern import six
 from ..utils import OrderedDict
+from ..utils.compat.numpy import broadcast_arrays
 
 __all__ = ["BaseRepresentation", "CartesianRepresentation",
            "SphericalRepresentation", "UnitSphericalRepresentation",
            "PhysicsSphericalRepresentation", "CylindricalRepresentation"]
 
+NUMPY_LT_1P7 = [int(x) for x in np.__version__.split('.')[:2]] < [1, 7]
+
 # Module-level dict mapping representation string alias names to class.
 # This is populated by the metaclass init so all representation classes
 # get registered automatically.
 REPRESENTATION_CLASSES = {}
 
-
-def broadcast_quantity(*args):
-    """
-    A Quantity-aware version of np.broadcast_arrays
-    """
-    new_arrays = np.broadcast_arrays(*args)
-    new_quantities = []
-    for i in range(len(new_arrays)):
-        new_quantities.append(args[i]._new_view(new_arrays[i]))
-    return tuple(new_quantities)
-
-
 class MetaBaseRepresentation(type):
     def __init__(cls, name, bases, dct):
         super(MetaBaseRepresentation, cls).__init__(name, bases, dct)
@@ -53,6 +45,17 @@ class MetaBaseRepresentation(type):
         REPRESENTATION_CLASSES[cls.get_name()] = cls
 
 
+def _fstyle(precision, x):
+    fmt_str = '{0:.{precision}f}'
+    s = fmt_str.format(x, precision=precision)
+    s_trunc = s.rstrip('0')
+    if s_trunc[-1] == '.':
+        # Ensure there is one trailing 0 after a bare decimal point
+        return s_trunc + '0'
+    else:
+        return s_trunc
+
+
 @six.add_metaclass(MetaBaseRepresentation)
 class BaseRepresentation(object):
     """
@@ -163,29 +166,42 @@ class BaseRepresentation(object):
         return unitstr
 
     def __str__(self):
-        if self.isscalar and len(set(self._units.values())) > 1:
-            return '({0})'.format(', '.join(
-                ['{0}'.format(getattr(self, component))
-                 for component in self.components]))
-        else:
-            return '{0} {1:s}'.format(self._values, self._unitstr)
+        return '{0} {1:s}'.format(self._values, self._unitstr)
 
     def __repr__(self):
-        if self.isscalar:
-            return '<{0} {1}>'.format(
-                self.__class__.__name__,
-                ', '.join(['{0}={1}'.format(component,
-                                            getattr(self, component))
-                           for component in self.components]))
+        prefixstr = '    '
 
+        if self._values.shape == ():
+            v = [tuple([self._values[nm] for nm in self._values.dtype.names])]
+            v = np.array(v, dtype=self._values.dtype)
         else:
-            prefixstr = '    '
-            arrstr = np.array2string(self._values, separator=', ',
+            v = self._values
+
+        names = self._values.dtype.names
+        precision = np.get_printoptions()['precision']
+        fstyle = functools.partial(_fstyle, precision)
+        format_val = lambda val: np.array2string(val, style=fstyle)
+        formatter = {
+            'numpystr': lambda x: '({0})'.format(
+                ', '.join(format_val(x[name]) for name in names))
+        }
+
+        if NUMPY_LT_1P7:
+            arrstr = np.array2string(v, separator=', ',
                                      prefix=prefixstr)
 
-            return '<{0} ({1}) in {2:s}\n{3}{4}>'.format(
-                self.__class__.__name__, ', '.join(self.components),
-                self._unitstr, prefixstr, arrstr)
+        else:
+            arrstr = np.array2string(v, formatter=formatter,
+                                     separator=', ',
+                                     prefix=prefixstr)
+
+        if self._values.shape == ():
+            arrstr = arrstr[1:-1]
+
+        unitstr = ('in ' + self._unitstr) if self._unitstr else '[dimensionless]'
+        return '<{0} ({1}) {2:s}\n{3}{4}>'.format(
+            self.__class__.__name__, ', '.join(self.components),
+            unitstr, prefixstr, arrstr)
 
 
 class CartesianRepresentation(BaseRepresentation):
@@ -230,7 +246,7 @@ class CartesianRepresentation(BaseRepresentation):
             raise u.UnitsError("x, y, and z should have matching physical types")
 
         try:
-            x, y, z = broadcast_quantity(x, y, z)
+            x, y, z = broadcast_arrays(x, y, z, subok=True)
         except ValueError:
             raise ValueError("Input parameters x, y, and z cannot be broadcast")
 
@@ -315,7 +331,8 @@ class SphericalRepresentation(BaseRepresentation):
             distance = distance.view(Distance)
 
         try:
-            lon, lat, distance = broadcast_quantity(lon, lat, distance)
+            lon, lat, distance = broadcast_arrays(lon, lat, distance,
+                                                  subok=True)
         except ValueError:
             raise ValueError("Input parameters lon, lat, and distance cannot be broadcast")
 
@@ -345,7 +362,7 @@ class SphericalRepresentation(BaseRepresentation):
         return self._distance
 
     def represent_as(self, other_class):
-        # Take a short cut if the other clsss is a spherical representation
+        # Take a short cut if the other class is a spherical representation
         if other_class is PhysicsSphericalRepresentation:
             return PhysicsSphericalRepresentation(phi=self.lon,
                                                   theta=90 * u.deg - self.lat,
@@ -422,7 +439,7 @@ class UnitSphericalRepresentation(BaseRepresentation):
         lat = self.attr_classes['lat'](lat, copy=copy)
 
         try:
-            lon, lat = broadcast_quantity(lon, lat)
+            lon, lat = broadcast_arrays(lon, lat, subok=True)
         except ValueError:
             raise ValueError("Input parameters lon and lat cannot be broadcast")
 
@@ -539,7 +556,7 @@ class PhysicsSphericalRepresentation(BaseRepresentation):
             r = r.view(Distance)
 
         try:
-            phi, theta, r = broadcast_quantity(phi, theta, r)
+            phi, theta, r = broadcast_arrays(phi, theta, r, subok=True)
         except ValueError:
             raise ValueError("Input parameters phi, theta, and r cannot be broadcast")
 
@@ -653,7 +670,7 @@ class CylindricalRepresentation(BaseRepresentation):
             raise u.UnitsError("rho and z should have matching physical types")
 
         try:
-            rho, phi, z = broadcast_quantity(rho, phi, z)
+            rho, phi, z = broadcast_arrays(rho, phi, z, subok=True)
         except ValueError:
             raise ValueError("Input parameters rho, phi, and z cannot be broadcast")
 
