@@ -16,12 +16,15 @@ import numpy as np
 from numpy import testing as npt
 
 from ... import units as u
-from ...tests.helper import pytest
+from ...tests.helper import pytest, catch_warnings
 from ..representation import REPRESENTATION_CLASSES
 from ...coordinates import (ICRS, FK4, FK5, Galactic, SkyCoord, Angle,
-                            SphericalRepresentation, CartesianRepresentation)
+                            SphericalRepresentation, CartesianRepresentation,
+                            UnitSphericalRepresentation)
 from ...coordinates import Latitude, Longitude
 from ...time import Time
+from ...utils import minversion
+from ...utils.exceptions import AstropyDeprecationWarning
 
 RA = 1.0 * u.deg
 DEC = 2.0 * u.deg
@@ -31,6 +34,17 @@ J2001 = Time('J2001', scale='utc')
 
 allclose = functools.partial(np.allclose, rtol=0.0, atol=1e-8)
 
+try:
+    import scipy
+    HAS_SCIPY = True
+except ImportError:
+    HAS_SCIPY = False
+
+if HAS_SCIPY and minversion(scipy, '0.12.0', inclusive=False):
+    OLDER_SCIPY = False
+else:
+    OLDER_SCIPY = True
+
 
 def test_transform_to():
     for frame in (FK5, FK5(equinox=Time('J1975.0')),
@@ -126,16 +140,65 @@ def test_coord_init_string():
 
     with pytest.raises(ValueError) as err:
         SkyCoord('1d 2d 3d')
-    assert "Cannot parse longitude and latitude" in str(err)
+    assert "Cannot parse first argument data" in str(err)
 
     sc1 = SkyCoord('8 00 00 +5 00 00.0', unit=(u.hour, u.deg), frame='icrs')
     assert isinstance(sc1, SkyCoord)
     assert allclose(sc1.ra, Angle(120 * u.deg))
     assert allclose(sc1.dec, Angle(5 * u.deg))
 
-    with pytest.raises(ValueError) as err:
-        SkyCoord('8 00 -5 00 00.0', unit=(u.hour, u.deg), frame='icrs')
-    assert 'coordinates have 5 values but spherical representation only accepts 3' in str(err)
+    sc11 = SkyCoord('8h00m00s+5d00m00.0s', unit=(u.hour, u.deg), frame='icrs')
+    assert isinstance(sc11, SkyCoord)
+    assert allclose(sc1.ra, Angle(120 * u.deg))
+    assert allclose(sc1.dec, Angle(5 * u.deg))
+
+    sc2 = SkyCoord('8 00 -5 00 00.0', unit=(u.hour, u.deg), frame='icrs')
+    assert isinstance(sc2, SkyCoord)
+    assert allclose(sc2.ra, Angle(120 * u.deg))
+    assert allclose(sc2.dec, Angle(-5 * u.deg))
+
+    sc3 = SkyCoord('8 00 -5 00.6', unit=(u.hour, u.deg), frame='icrs')
+    assert isinstance(sc3, SkyCoord)
+    assert allclose(sc3.ra, Angle(120 * u.deg))
+    assert allclose(sc3.dec, Angle(-5.01 * u.deg))
+
+    sc4 = SkyCoord('J080000.00-050036.00', unit=(u.hour, u.deg), frame='icrs')
+    assert isinstance(sc4, SkyCoord)
+    assert allclose(sc4.ra, Angle(120 * u.deg))
+    assert allclose(sc4.dec, Angle(-5.01 * u.deg))
+
+    sc41 = SkyCoord('J080000+050036', unit=(u.hour, u.deg), frame='icrs')
+    assert isinstance(sc41, SkyCoord)
+    assert allclose(sc41.ra, Angle(120 * u.deg))
+    assert allclose(sc41.dec, Angle(+5.01 * u.deg))
+
+    sc5 = SkyCoord('8h00.6m -5d00.6m', unit=(u.hour, u.deg), frame='icrs')
+    assert isinstance(sc5, SkyCoord)
+    assert allclose(sc5.ra, Angle(120.15 * u.deg))
+    assert allclose(sc5.dec, Angle(-5.01 * u.deg))
+
+    sc6 = SkyCoord('8h00.6m -5d00.6m', unit=(u.hour, u.deg), frame='fk4')
+    assert isinstance(sc6, SkyCoord)
+    assert allclose(sc6.ra, Angle(120.15 * u.deg))
+    assert allclose(sc6.dec, Angle(-5.01 * u.deg))
+
+    sc61 = SkyCoord('8h00.6m-5d00.6m', unit=(u.hour, u.deg), frame='fk4')
+    assert isinstance(sc61, SkyCoord)
+    assert allclose(sc6.ra, Angle(120.15 * u.deg))
+    assert allclose(sc6.dec, Angle(-5.01 * u.deg))
+
+    sc61 = SkyCoord('8h00.6-5d00.6', unit=(u.hour, u.deg), frame='fk4')
+    assert isinstance(sc61, SkyCoord)
+    assert allclose(sc6.ra, Angle(120.15 * u.deg))
+    assert allclose(sc6.dec, Angle(-5.01 * u.deg))
+
+    sc7 = SkyCoord("J1874221.60+122421.6", unit=u.deg)
+    assert isinstance(sc7, SkyCoord)
+    assert allclose(sc7.ra, Angle(187.706 * u.deg))
+    assert allclose(sc7.dec, Angle(12.406 * u.deg))
+
+    with pytest.raises(ValueError):
+        SkyCoord('8 00 -5 00.6', unit=(u.deg, u.deg), frame='galactic')
 
 
 def test_coord_init_unit():
@@ -183,11 +246,11 @@ def test_coord_init_list():
 
     with pytest.raises(ValueError) as err:
         SkyCoord(['1d 2d 3d'])
-    assert "Cannot parse longitude and latitude" in str(err)
+    assert "Cannot parse first argument data" in str(err)
 
     with pytest.raises(ValueError) as err:
         SkyCoord([('1d', '2d', '3d')])
-    assert "Cannot parse longitude and latitude" in str(err)
+    assert "Cannot parse first argument data" in str(err)
 
     sc = SkyCoord([1 * u.deg, 1 * u.deg], [2 * u.deg, 2 * u.deg])
     assert allclose(sc.ra, Angle('1d'))
@@ -236,27 +299,44 @@ def test_coord_init_representation():
     assert allclose(sc_cart.z, 3.0)
 
 
+FRAME_DEPRECATION_WARNING = ("Passing a frame as a positional argument is now "
+                             "deprecated, use the frame= keyword argument "
+                             "instead.")
+
 def test_frame_init():
     """
     Different ways of providing the frame.
     """
+
     sc = SkyCoord(RA, DEC, frame='icrs')
     assert sc.frame.name == 'icrs'
 
     sc = SkyCoord(RA, DEC, frame=ICRS)
     assert sc.frame.name == 'icrs'
 
-    sc = SkyCoord(RA, DEC, 'icrs')
+    with catch_warnings(AstropyDeprecationWarning) as w:
+        sc = SkyCoord(RA, DEC, 'icrs')
     assert sc.frame.name == 'icrs'
+    assert len(w) == 1
+    assert str(w[0].message) == FRAME_DEPRECATION_WARNING
 
-    sc = SkyCoord(RA, DEC, ICRS)
+    with catch_warnings(AstropyDeprecationWarning) as w:
+        sc = SkyCoord(RA, DEC, ICRS)
     assert sc.frame.name == 'icrs'
+    assert len(w) == 1
+    assert str(w[0].message) == FRAME_DEPRECATION_WARNING
 
-    sc = SkyCoord('icrs', RA, DEC)
+    with catch_warnings(AstropyDeprecationWarning) as w:
+        sc = SkyCoord('icrs', RA, DEC)
     assert sc.frame.name == 'icrs'
+    assert len(w) == 1
+    assert str(w[0].message) == FRAME_DEPRECATION_WARNING
 
-    sc = SkyCoord(ICRS, RA, DEC)
+    with catch_warnings(AstropyDeprecationWarning) as w:
+        sc = SkyCoord(ICRS, RA, DEC)
     assert sc.frame.name == 'icrs'
+    assert len(w) == 1
+    assert str(w[0].message) == FRAME_DEPRECATION_WARNING
 
     sc = SkyCoord(sc)
     assert sc.frame.name == 'icrs'
@@ -278,7 +358,7 @@ def test_attr_inheritance():
     equinox should be inherited to the SkyCoord.  If there is a conflict
     then raise an exception.
     """
-    sc = SkyCoord('icrs', 1, 2, unit='deg', equinox='J1999', obstime='J2001')
+    sc = SkyCoord(1, 2, frame='icrs', unit='deg', equinox='J1999', obstime='J2001')
     sc2 = SkyCoord(sc)
     assert sc2.equinox == sc.equinox
     assert sc2.obstime == sc.obstime
@@ -293,7 +373,7 @@ def test_attr_inheritance():
     assert allclose(sc2.dec, sc.dec)
     assert allclose(sc2.distance, sc.distance)
 
-    sc = SkyCoord('fk4', 1, 2, unit='deg', equinox='J1999', obstime='J2001')
+    sc = SkyCoord(1, 2, frame='fk4', unit='deg', equinox='J1999', obstime='J2001')
     sc2 = SkyCoord(sc)
     assert sc2.equinox == sc.equinox
     assert sc2.obstime == sc.obstime
@@ -313,7 +393,7 @@ def test_attr_conflicts():
     """
     Check conflicts resolution between coordinate attributes and init kwargs.
     """
-    sc = SkyCoord('icrs', 1, 2, unit='deg', equinox='J1999', obstime='J2001')
+    sc = SkyCoord(1, 2, frame='icrs', unit='deg', equinox='J1999', obstime='J2001')
 
     # OK if attrs both specified but with identical values
     SkyCoord(sc, equinox='J1999', obstime='J2001')
@@ -327,7 +407,7 @@ def test_attr_conflicts():
     assert "Coordinate attribute 'obstime'=" in str(err)
 
     # Same game but with fk4 which has equinox and obstime frame attrs
-    sc = SkyCoord('fk4', 1, 2, unit='deg', equinox='J1999', obstime='J2001')
+    sc = SkyCoord(1, 2, frame='fk4', unit='deg', equinox='J1999', obstime='J2001')
 
     # OK if attrs both specified but with identical values
     SkyCoord(sc, equinox='J1999', obstime='J2001')
@@ -349,15 +429,15 @@ def test_frame_attr_getattr():
     from self.frame when that object has the relevant attribute, otherwise
     from self.
     """
-    sc = SkyCoord('icrs', 1, 2, unit='deg', equinox='J1999', obstime='J2001')
+    sc = SkyCoord(1, 2, frame='icrs', unit='deg', equinox='J1999', obstime='J2001')
     assert sc.equinox == 'J1999'  # Just the raw value (not validated)
     assert sc.obstime == 'J2001'
 
-    sc = SkyCoord('fk4', 1, 2, unit='deg', equinox='J1999', obstime='J2001')
+    sc = SkyCoord(1, 2, frame='fk4', unit='deg', equinox='J1999', obstime='J2001')
     assert sc.equinox == Time('J1999')  # Coming from the self.frame object
     assert sc.obstime == Time('J2001')
 
-    sc = SkyCoord('fk4', 1, 2, unit='deg', equinox='J1999')
+    sc = SkyCoord(1, 2, frame='fk4', unit='deg', equinox='J1999')
     assert sc.equinox == Time('J1999')
     assert sc.obstime == Time('J1999')
 
@@ -379,8 +459,8 @@ def test_to_string():
 
 
 def test_seps():
-    sc1 = SkyCoord('icrs', 0 * u.deg, 1 * u.deg)
-    sc2 = SkyCoord('icrs', 0 * u.deg, 2 * u.deg)
+    sc1 = SkyCoord(0 * u.deg, 1 * u.deg, frame='icrs')
+    sc2 = SkyCoord(0 * u.deg, 2 * u.deg, frame='icrs')
 
     sep = sc1.separation(sc2)
 
@@ -389,8 +469,8 @@ def test_seps():
     with pytest.raises(ValueError):
         sc1.separation_3d(sc2)
 
-    sc3 = SkyCoord('icrs', 1 * u.deg, 1 * u.deg, distance=1 * u.kpc)
-    sc4 = SkyCoord('icrs', 1 * u.deg, 1 * u.deg, distance=2 * u.kpc)
+    sc3 = SkyCoord(1 * u.deg, 1 * u.deg, distance=1 * u.kpc, frame='icrs')
+    sc4 = SkyCoord(1 * u.deg, 1 * u.deg, distance=2 * u.kpc, frame='icrs')
     sep3d = sc3.separation_3d(sc4)
 
     assert sep3d == 1 * u.kpc
@@ -399,27 +479,30 @@ def test_seps():
 def test_repr():
     # Repr tests must use exact floating point vals because Python 2.6
     # outputs values like 0.1 as 0.1000000000001.  No workaround found.
-    sc1 = SkyCoord('icrs', 0 * u.deg, 1 * u.deg)
-    sc2 = SkyCoord('icrs', 1 * u.deg, 1 * u.deg, distance=1 * u.kpc)
+    sc1 = SkyCoord(0 * u.deg, 1 * u.deg, frame='icrs')
+    sc2 = SkyCoord(1 * u.deg, 1 * u.deg, frame='icrs', distance=1 * u.kpc)
 
-    assert repr(sc1) == '<SkyCoord (ICRS): ra=0.0 deg, dec=1.0 deg>'
-    assert repr(sc2) == '<SkyCoord (ICRS): ra=1.0 deg, dec=1.0 deg, distance=1.0 kpc>'
+    assert repr(sc1) == ('<SkyCoord (ICRS): (ra, dec) in deg\n'
+                         '    (0.0, 1.0)>')
+    assert repr(sc2) == ('<SkyCoord (ICRS): (ra, dec, distance) in (deg, deg, kpc)\n'
+                         '    (1.0, 1.0, 1.0)>')
 
-    sc3 = SkyCoord('icrs', 0.25 * u.deg, [1, 2.5] * u.deg)
+    sc3 = SkyCoord(0.25 * u.deg, [1, 2.5] * u.deg, frame='icrs')
     assert repr(sc3) == ('<SkyCoord (ICRS): (ra, dec) in deg\n'
                          '    [(0.25, 1.0), (0.25, 2.5)]>')
 
     sc_default = SkyCoord(0 * u.deg, 1 * u.deg)
-    assert repr(sc_default) == '<SkyCoord (ICRS): ra=0.0 deg, dec=1.0 deg>'
+    assert repr(sc_default) == ('<SkyCoord (ICRS): (ra, dec) in deg\n'
+                                '    (0.0, 1.0)>')
 
 
 def test_ops():
     """
     Tests miscellaneous operations like `len`
     """
-    sc = SkyCoord('icrs', 0 * u.deg, 1 * u.deg)
-    sc_arr = SkyCoord('icrs', 0 * u.deg, [1, 2] * u.deg)
-    sc_empty = SkyCoord('icrs', [] * u.deg, [] * u.deg)
+    sc = SkyCoord(0 * u.deg, 1 * u.deg, frame='icrs')
+    sc_arr = SkyCoord(0 * u.deg, [1, 2] * u.deg, frame='icrs')
+    sc_empty = SkyCoord([] * u.deg, [] * u.deg, frame='icrs')
 
     assert sc.isscalar
     assert not sc_arr.isscalar
@@ -706,11 +789,46 @@ def test_units_known_fail():
     with pytest.raises(u.UnitsError):
         SkyCoord(1, 2, 3, unit=u.deg, representation='spherical')
 
+
 def test_nodata_failure():
     with pytest.raises(ValueError):
         SkyCoord()
 
 
+@pytest.mark.parametrize('mode, origin', [('wcs', 0),
+                                          ('all', 0),
+                                          ('all', 1)])
+def test_wcs_methods(mode, origin):
+    from ...wcs import WCS
+    from ...utils.data import get_pkg_data_contents
+    from ...wcs.utils import pixel_to_skycoord
+
+    header = get_pkg_data_contents('../../wcs/tests/maps/1904-66_TAN.hdr', encoding='binary')
+    wcs = WCS(header)
+
+    ref = SkyCoord(0.1 * u.deg, -89. * u.deg, frame='icrs')
+
+    xp, yp = ref.to_pixel(wcs, mode=mode, origin=origin)
+
+    # WCS is in FK5 so we need to transform back to ICRS
+    new = pixel_to_skycoord(xp, yp, wcs, mode=mode, origin=origin).transform_to('icrs')
+
+    npt.assert_allclose(new.ra.degree, ref.ra.degree)
+    npt.assert_allclose(new.dec.degree, ref.dec.degree)
+
+    #also try to round-trip with `from_pixel`
+    scnew = SkyCoord.from_pixel(xp, yp, wcs, mode=mode, origin=origin).transform_to('icrs')
+    npt.assert_allclose(scnew.ra.degree, ref.ra.degree)
+    npt.assert_allclose(scnew.dec.degree, ref.dec.degree)
+
+    #Also make sure the right type comes out
+    class SkyCoord2(SkyCoord):
+        pass
+    scnew2 = SkyCoord2.from_pixel(xp, yp, wcs, mode=mode, origin=origin)
+    assert scnew.__class__ is SkyCoord
+    assert scnew2.__class__ is SkyCoord2
+
+
 def test_frame_attr_transform_inherit():
     """
     Test that frame attributes get inherited as expected during transform.
@@ -767,3 +885,228 @@ def test_immutable():
 
     c1.foo = 42
     assert c1.foo == 42
+
+
+@pytest.mark.skipif(str('not HAS_SCIPY'))
+@pytest.mark.skipif(str('OLDER_SCIPY'))
+def test_search_around():
+    """
+    Test the search_around_* methods
+
+    Here we don't actually test the values are right, just that the methods of
+    SkyCoord work.  The accuracy tests are in ``test_matching.py``
+    """
+    from ...utils import NumpyRNGContext
+
+    with NumpyRNGContext(987654321):
+        sc1 = SkyCoord(np.random.rand(20) * 360.*u.degree,
+                      (np.random.rand(20) * 180. - 90.)*u.degree)
+        sc2 = SkyCoord(np.random.rand(100) * 360. * u.degree,
+                      (np.random.rand(100) * 180. - 90.)*u.degree)
+
+        sc1ds = SkyCoord(ra=sc1.ra, dec=sc1.dec, distance=np.random.rand(20)*u.kpc)
+        sc2ds = SkyCoord(ra=sc2.ra, dec=sc2.dec, distance=np.random.rand(100)*u.kpc)
+
+    idx1_sky, idx2_sky, d2d_sky, d3d_sky = sc1.search_around_sky(sc2, 10*u.deg)
+    idx1_3d, idx2_3d, d2d_3d, d3d_3d = sc1ds.search_around_3d(sc2ds, 250*u.pc)
+
+
+def test_init_with_frame_instance_keyword():
+
+    # Frame instance
+    c1 = SkyCoord(3 * u.deg, 4 * u.deg,
+                  frame=FK5(equinox='J2010'))
+    assert c1.equinox == Time('J2010')
+
+    # Frame instance with data (data gets ignored)
+    c2 = SkyCoord(3 * u.deg, 4 * u.deg,
+                 frame=FK5(1. * u.deg, 2 * u.deg,
+                 equinox='J2010'))
+    assert c2.equinox == Time('J2010')
+    assert allclose(c2.ra.degree, 3)
+    assert allclose(c2.dec.degree, 4)
+
+    # SkyCoord instance
+    c3 = SkyCoord(3 * u.deg, 4 * u.deg, frame=c1)
+    assert c3.equinox == Time('J2010')
+
+    # Check duplicate arguments
+    with pytest.raises(ValueError) as exc:
+        c = SkyCoord(3 * u.deg, 4 * u.deg, frame=FK5(equinox='J2010'), equinox='J2001')
+    assert exc.value.args[0] == ("cannot specify frame attribute "
+                                 "'equinox' directly in SkyCoord "
+                                 "since a frame instance was passed in")
+
+
+def test_init_with_frame_instance_positional():
+
+    # Frame instance
+    with pytest.raises(ValueError) as exc:
+        c1 = SkyCoord(3 * u.deg, 4 * u.deg, FK5(equinox='J2010'))
+    assert exc.value.args[0] == ("FK5 instance cannot be passed as a "
+                                 "positional argument for the frame, "
+                                 "pass it using the frame= keyword "
+                                 "instead.")
+
+    # Positional frame instance with data raises exception
+    with pytest.raises(ValueError) as exc:
+        SkyCoord(3 * u.deg, 4 * u.deg, FK5(1. * u.deg, 2 * u.deg, equinox='J2010'))
+    assert exc.value.args[0] == ("FK5 instance cannot be passed as a "
+                                 "positional argument for the frame, "
+                                 "pass it using the frame= keyword "
+                                 "instead.")
+
+    # Positional SkyCoord instance (for frame) raises exception
+    with pytest.raises(ValueError) as exc:
+        SkyCoord(3 * u.deg, 4 * u.deg, SkyCoord(1. * u.deg, 2 * u.deg, equinox='J2010'))
+    assert exc.value.args[0] == ("SkyCoord instance cannot be passed as a "
+                                 "positional argument for the frame, "
+                                 "pass it using the frame= keyword "
+                                 "instead.")
+
+
+def test_guess_from_table():
+    from ...table import Table, Column
+    from ...utils import NumpyRNGContext
+
+    tab = Table()
+    with NumpyRNGContext(987654321):
+        tab.add_column(Column(data=np.random.rand(1000),unit='deg',name='RA[J2000]'))
+        tab.add_column(Column(data=np.random.rand(1000),unit='deg',name='DEC[J2000]'))
+
+    sc = SkyCoord.guess_from_table(tab)
+    npt.assert_array_equal(sc.ra.deg, tab['RA[J2000]'])
+    npt.assert_array_equal(sc.dec.deg, tab['DEC[J2000]'])
+
+    # try without units in the table
+    tab['RA[J2000]'].unit = None
+    tab['DEC[J2000]'].unit = None
+    # should fail if not given explicitly
+    with pytest.raises(u.UnitsError):
+        sc2 = SkyCoord.guess_from_table(tab)
+
+    # but should work if provided
+    sc2 = SkyCoord.guess_from_table(tab, unit=u.deg)
+    npt.assert_array_equal(sc.ra.deg, tab['RA[J2000]'])
+    npt.assert_array_equal(sc.dec.deg, tab['DEC[J2000]'])
+
+    # should fail if two options are available - ambiguity bad!
+    tab.add_column(Column(data=np.random.rand(1000), name='RA_J1900'))
+    with pytest.raises(ValueError) as excinfo:
+        sc3 = SkyCoord.guess_from_table(tab, unit=u.deg)
+    assert 'J1900' in excinfo.value.args[0] and 'J2000' in excinfo.value.args[0]
+
+    # should also fail if user specifies something already in the table, but
+    # should succeed even if the user has to give one of the components
+    tab.remove_column('RA_J1900')
+    with pytest.raises(ValueError):
+        sc3 = SkyCoord.guess_from_table(tab, ra=tab['RA[J2000]'], unit=u.deg)
+
+    oldra = tab['RA[J2000]']
+    tab.remove_column('RA[J2000]')
+    sc3 = SkyCoord.guess_from_table(tab, ra=oldra, unit=u.deg)
+    npt.assert_array_equal(sc3.ra.deg, oldra)
+    npt.assert_array_equal(sc3.dec.deg, tab['DEC[J2000]'])
+
+    # check a few non-ICRS/spherical systems
+    x, y, z = np.arange(3).reshape(3, 1) * u.pc
+    l, b = np.arange(2).reshape(2, 1) * u.deg
+
+    tabcart = Table([x, y, z], names=('x', 'y', 'z'))
+    tabgal = Table([b, l], names=('b', 'l'))
+
+    sc_cart = SkyCoord.guess_from_table(tabcart, representation='cartesian')
+    npt.assert_array_equal(sc_cart.x, x)
+    npt.assert_array_equal(sc_cart.y, y)
+    npt.assert_array_equal(sc_cart.z, z)
+
+    sc_gal = SkyCoord.guess_from_table(tabgal, frame='galactic')
+    npt.assert_array_equal(sc_gal.l, l)
+    npt.assert_array_equal(sc_gal.b, b)
+
+    #also try some column names that *end* with the attribute name
+    tabgal['b'].name = 'gal_b'
+    tabgal['l'].name = 'gal_l'
+    SkyCoord.guess_from_table(tabgal, frame='galactic')
+
+    tabgal['gal_b'].name = 'blob'
+    tabgal['gal_l'].name = 'central'
+    with pytest.raises(ValueError):
+        SkyCoord.guess_from_table(tabgal, frame='galactic')
+
+
+def test_skycoord_list_creation():
+    """
+    Test that SkyCoord can be created in a reasonable way with lists of SkyCoords
+    (regression for #2702)
+    """
+    sc = SkyCoord(ra=[1, 2, 3]*u.deg, dec=[4, 5, 6]*u.deg)
+    sc0 = sc[0]
+    sc2 = sc[2]
+    scnew = SkyCoord([sc0, sc2])
+    assert np.all(scnew.ra == [1, 3]*u.deg)
+    assert np.all(scnew.dec == [4, 6]*u.deg)
+
+    #also check ranges
+    sc01 = sc[:2]
+    scnew2 = SkyCoord([sc01, sc2])
+    assert np.all(scnew2.ra == sc.ra)
+    assert np.all(scnew2.dec == sc.dec)
+
+    #now try with a mix of skycoord, frame, and repr objects
+    frobj = ICRS(2*u.deg, 5*u.deg)
+    reprobj = UnitSphericalRepresentation(3*u.deg, 6*u.deg)
+    scnew3 = SkyCoord([sc0, frobj, reprobj])
+    assert np.all(scnew3.ra == sc.ra)
+    assert np.all(scnew3.dec == sc.dec)
+
+    #should *fail* if different frame attributes or types are passed in
+    scfk5_j2000 = SkyCoord(1*u.deg, 4*u.deg, frame='fk5')
+    with pytest.raises(ValueError):
+        SkyCoord([sc0, scfk5_j2000])
+    scfk5_j2010 = SkyCoord(1*u.deg, 4*u.deg, frame='fk5', equinox='J2010')
+    with pytest.raises(ValueError):
+        SkyCoord([scfk5_j2000, scfk5_j2010])
+
+    # but they should inherit if they're all consistent
+    scfk5_2_j2010 = SkyCoord(2*u.deg, 5*u.deg, frame='fk5', equinox='J2010')
+    scfk5_3_j2010 = SkyCoord(3*u.deg, 6*u.deg, frame='fk5', equinox='J2010')
+
+    scnew4 = SkyCoord([scfk5_j2010, scfk5_2_j2010, scfk5_3_j2010])
+    assert np.all(scnew4.ra == sc.ra)
+    assert np.all(scnew4.dec == sc.dec)
+    assert scnew4.equinox == Time('J2010')
+
+
+def test_nd_skycoord_to_string():
+    c = SkyCoord(np.ones((2, 2)), 1, unit=('deg', 'deg'))
+    ts = c.to_string()
+    assert np.all(ts.shape == c.shape)
+    assert np.all(ts == u'1 1')
+
+
+def test_equiv_skycoord():
+    sci1 = SkyCoord(1*u.deg, 2*u.deg, frame='icrs')
+    sci2 = SkyCoord(1*u.deg, 3*u.deg, frame='icrs')
+    assert sci1.is_equivalent_frame(sci1)
+    assert sci1.is_equivalent_frame(sci2)
+
+    assert sci1.is_equivalent_frame(ICRS())
+    assert not sci1.is_equivalent_frame(FK5())
+    with pytest.raises(TypeError):
+        sci1.is_equivalent_frame(10)
+
+    scf1 = SkyCoord(1*u.deg, 2*u.deg, frame='fk5')
+    scf2 = SkyCoord(1*u.deg, 2*u.deg, frame='fk5', equinox='J2005')
+    #obstime is *not* an FK5 attribute, but we still want scf1 and scf3 to come
+    #to come out different because they're part of SkyCoord
+    scf3 = SkyCoord(1*u.deg, 2*u.deg, frame='fk5', obstime='J2005')
+
+    assert scf1.is_equivalent_frame(scf1)
+    assert not scf1.is_equivalent_frame(sci1)
+    assert scf1.is_equivalent_frame(FK5())
+
+    assert not scf1.is_equivalent_frame(scf2)
+    assert scf2.is_equivalent_frame(FK5(equinox='J2005'))
+    assert not scf3.is_equivalent_frame(scf1)
+    assert not scf3.is_equivalent_frame(FK5(equinox='J2005'))
