@@ -1,6 +1,19 @@
 # Licensed under a 3-clause BSD style license
 from __future__ import absolute_import
 
+import os
+from distutils.extension import Extension
+
+ROOT = os.path.relpath(os.path.dirname(__file__))
+
+def get_extensions():
+    sources = [os.path.join(ROOT, 'cparser.pyx'),
+               os.path.join(ROOT, 'src', 'tokenizer.c')]
+    ascii_ext = Extension(
+        name="astropy.io.ascii.cparser",
+        include_dirs=["numpy"],
+        sources=sources)
+    return [ascii_ext]
 
 def get_package_data():
     # Installs the testing data files.  Unable to get package_data
@@ -63,6 +76,7 @@ def get_package_data():
                                    't/whitespace.dat',
                                    't/simple_csv.csv',
                                    't/simple_csv_missing.csv',
+                                   't/fixed_width_2_line.txt',
                                    ]
     }
 
