@@ -51,7 +51,7 @@ explanation of all the different formats.
     multiple argument formats that were used in past versions of PyFITS.
     Unfortunately, it is not possible to support all formats without
     introducing some ambiguity.  A future Astropy release may standardize
-    around a single format and offically deprecate the other formats.
+    around a single format and officially deprecate the other formats.
 """
 
 
@@ -65,8 +65,8 @@ from .hdu.hdulist import fitsopen
 from .hdu.image import PrimaryHDU, ImageHDU
 from .hdu.table import BinTableHDU
 from .header import Header
-from .util import (fileobj_closed, fileobj_name, fileobj_mode,
-                   fileobj_closed, _is_int)
+from .util import fileobj_closed, fileobj_name, fileobj_mode, _is_int
+from ...extern.six import string_types
 from ...utils import deprecated
 
 
@@ -154,7 +154,7 @@ def getdata(filename, *args, **kwargs):
         tuple.
 
     lower, upper : bool, optional
-        If `lower` or `upper` are `True`, the field names in the
+        If ``lower`` or ``upper`` are `True`, the field names in the
         returned data object will be converted to lower or upper case,
         respectively.
 
@@ -382,14 +382,16 @@ def writeto(filename, data, header=None, output_verify='exception',
         data to write to the new file
 
     header : `Header` object, optional
-        the header associated with `data`. If `None`, a header
+        the header associated with ``data``. If `None`, a header
         of the appropriate type is created for the supplied data. This
         argument is optional.
 
     output_verify : str
-        Output verification option.  Must be one of ``"fix"``,
-        ``"silentfix"``, ``"ignore"``, ``"warn"``, or
-        ``"exception"``.  See :ref:`verify` for more info.
+        Output verification option.  Must be one of ``"fix"``, ``"silentfix"``,
+        ``"ignore"``, ``"warn"``, or ``"exception"``.  May also be any
+        combination of ``"fix"`` or ``"silentfix"`` with ``"+ignore"``,
+        ``+warn``, or ``+exception" (e.g. ``"fix+warn"``).  See :ref:`verify`
+        for more info.
 
     clobber : bool, optional
         If `True`, and if filename already exists, it will overwrite
@@ -411,7 +413,7 @@ def append(filename, data, header=None, checksum=False, verify=True, **kwargs):
     """
     Append the header/data to FITS file if filename exists, create if not.
 
-    If only `data` is supplied, a minimal header is created.
+    If only ``data`` is supplied, a minimal header is created.
 
     Parameters
     ----------
@@ -431,7 +433,7 @@ def append(filename, data, header=None, checksum=False, verify=True, **kwargs):
         When `True` adds both ``DATASUM`` and ``CHECKSUM`` cards to the header
         of the HDU when written to the file.
 
-    verify: bool, optional
+    verify : bool, optional
         When `True`, the existing FITS file will be read in to verify it for
         correctness before appending.  When `False`, content is simply appended
         to the end of the file.  Setting ``verify`` to `False` can be much
@@ -508,7 +510,7 @@ def update(filename, data, *args, **kwargs):
         `astropy.io.fits.open`.
     """
 
-    # The arguments to this function are a bit tricker to deal with than others
+    # The arguments to this function are a bit trickier to deal with than others
     # in this module, since the documentation has promised that the header
     # argument can be an optional positional argument.
     if args and isinstance(args[0], Header):
@@ -604,7 +606,7 @@ def tabledump(filename, datafile=None, cdfile=None, hfile=None, ext=1,
     -----
     The primary use for the `tabledump` function is to allow editing in a
     standard text editor of the table data and parameters.  The
-    `tcreate` function can be used to reassemble the table from the
+    ``tcreate`` function can be used to reassemble the table from the
     three ASCII files.
     """
 
@@ -704,7 +706,7 @@ def _getext(filename, mode, *args, **kwargs):
             if ext is not None or extname is not None or extver is not None:
                 raise TypeError(err_msg)
             ext = args[0]
-        elif isinstance(args[0], basestring):
+        elif isinstance(args[0], string_types):
             # The first arg is an extension name; it could still be valid
             # to provide an extver kwarg
             if ext is not None or extname is not None:
@@ -726,11 +728,11 @@ def _getext(filename, mode, *args, **kwargs):
     if (ext is not None and
             not (_is_int(ext) or
                  (isinstance(ext, tuple) and len(ext) == 2 and
-                  isinstance(ext[0], basestring) and _is_int(ext[1])))):
+                  isinstance(ext[0], string_types) and _is_int(ext[1])))):
         raise ValueError(
             'The ext keyword must be either an extension number '
             '(zero-indexed) or a (extname, extver) tuple.')
-    if extname is not None and not isinstance(extname, basestring):
+    if extname is not None and not isinstance(extname, string_types):
         raise ValueError('The extname argument must be a string.')
     if extver is not None and not _is_int(extver):
         raise ValueError('The extver argument must be an integer.')
