@@ -13,16 +13,20 @@ import numpy as np
 from numpy import char as chararray
 
 from .base import DELAYED, _ValidHDU, ExtensionHDU
+# This module may have many dependencies on pyfits.column, but pyfits.column
+# has fewer dependencies overall, so it's easier to keep table/column-related
+# utilities in pyfits.column
 from ..column import (FITS2NUMPY, KEYWORD_NAMES, KEYWORD_ATTRIBUTES, TDEF_RE,
-                      Column, ColDefs, _AsciiColDefs, _FormatX, _FormatP,
-                      _FormatQ, _makep, _VLF, _parse_tformat,
-                      _scalar_to_format, _convert_format, _cmp_recformats,
-                      _get_index)
+                      Column, ColDefs, _AsciiColDefs, _FormatP, _FormatQ,
+                      _makep, _parse_tformat, _scalar_to_format,
+                      _convert_format, _cmp_recformats, _get_index)
 from ..fitsrec import FITS_rec
 from ..header import Header, _pad_length
 from ..util import _is_int, _str_to_num
 
+from ....extern.six import string_types
 from ....utils import deprecated, lazyproperty
+from ....utils.compat import ignored
 from ....utils.exceptions import AstropyUserWarning
 
 
@@ -42,7 +46,7 @@ class _TableLikeHDU(_ValidHDU):
     """
     A class for HDUs that have table-like data.  This is used for both
     Binary/ASCII tables as well as Random Access Group HDUs (which are
-    otherwise too dissimlary for tables to use _TableBaseHDU directly).
+    otherwise too dissimilar for tables to use _TableBaseHDU directly).
     """
 
     _data_type = FITS_rec
@@ -62,6 +66,61 @@ class _TableLikeHDU(_ValidHDU):
 
         raise NotImplementedError
 
+    @classmethod
+    def from_columns(cls, columns, header=None, nrows=0, fill=False,
+                     **kwargs):
+        """
+        Given either a `ColDefs` object, a sequence of `Column` objects,
+        or another table HDU or table data (a `FITS_rec` or multi-field
+        `numpy.ndarray` or `numpy.recarray` object, return a new table HDU of
+        the class this method was called on using the column definition from
+        the input.
+
+        This is an alternative to the now deprecated `new_table` function,
+        and otherwise accepts the same arguments.  See also
+        `FITS_rec.from_columns`.
+
+        Parameters
+        ----------
+        columns : sequence of `Column`, `ColDefs`, or other
+            The columns from which to create the table data, or an object with
+            a column-like structure from which a `ColDefs` can be instantiated.
+            This includes an existing `BinTableHDU` or `TableHDU`, or a
+            `numpy.recarray` to give some examples.
+
+            If these columns have data arrays attached that data may be used in
+            initializing the new table.  Otherwise the input columns will be
+            used as a template for a new table with the requested number of
+            rows.
+
+        header : `Header`
+            An optional `Header` object to instantiate the new HDU yet.  Header
+            keywords specifically related to defining the table structure (such
+            as the "TXXXn" keywords like TTYPEn) will be overridden by the
+            supplied column definitions, but all other informational and data
+            model-specific keywords are kept.
+
+        nrows : int
+            Number of rows in the new table.  If the input columns have data
+            associated with them, the size of the largest input column is used.
+            Otherwise the default is 0.
+
+        fill : bool
+            If `True`, will fill all cells with zeros or blanks.  If `False`,
+            copy the data from input, undefined cells will still be filled with
+            zeros/blanks.
+
+        Notes
+        -----
+
+        Any additional keyword arguments accepted by the HDU class's
+        ``__init__`` may also be passed in as keyword arguments.
+        """
+
+        coldefs = cls._columns_type(columns)
+        data = FITS_rec.from_columns(coldefs, nrows=nrows, fill=fill)
+        return cls(data=data, header=header, **kwargs)
+
     @lazyproperty
     def columns(self):
         """
@@ -128,6 +187,16 @@ class _TableBaseHDU(ExtensionHDU, _TableLikeHDU):
     FITS table extension base HDU class.
     """
 
+    _manages_own_heap = False
+    """
+    This flag implies that when writing VLA tables (P/Q format) the heap
+    pointers that go into P/Q table columns should not be reordered or
+    rearranged in any way by the default heap management code.
+
+    This is included primarily as an optimization for compressed image HDUs
+    which perform their own heap maintenance.
+    """
+
     def __init__(self, data=None, header=None, name=None, uint=False):
         """
         Parameters
@@ -232,26 +301,22 @@ class _TableBaseHDU(ExtensionHDU, _TableLikeHDU):
                 self.columns = self.data._coldefs
                 self.update()
 
-                try:
-                   # Make the ndarrays in the Column objects of the ColDefs
-                   # object of the HDU reference the same ndarray as the HDU's
-                   # FITS_rec object.
+                with ignored(TypeError, AttributeError):
+                    # Make the ndarrays in the Column objects of the ColDefs
+                    # object of the HDU reference the same ndarray as the HDU's
+                    # FITS_rec object.
                     for idx in range(len(self.columns)):
                         self.columns[idx].array = self.data.field(idx)
 
                     # Delete the _arrays attribute so that it is recreated to
                     # point to the new data placed in the column objects above
                     del self.columns._arrays
-                except (TypeError, AttributeError) as e:
-                    # This shouldn't happen as long as self.columns._arrays
-                    # is a lazyproperty
-                    pass
             elif data is None:
                 pass
             else:
                 raise TypeError('Table data has incorrect type.')
 
-        if not (isinstance(self._header[0], basestring) and
+        if not (isinstance(self._header[0], string_types) and
                 self._header[0].rstrip() == self._extension):
             self._header[0] = (self._extension, self._ext_comment)
 
@@ -326,20 +391,16 @@ class _TableBaseHDU(ExtensionHDU, _TableLikeHDU):
             self.columns = self.data.columns
             self.update()
 
-            try:
-               # Make the ndarrays in the Column objects of the ColDefs
-               # object of the HDU reference the same ndarray as the HDU's
-               # FITS_rec object.
+            with ignored(TypeError, AttributeError):
+                # Make the ndarrays in the Column objects of the ColDefs
+                # object of the HDU reference the same ndarray as the HDU's
+                # FITS_rec object.
                 for idx in range(len(self.columns)):
                     self.columns[idx].array = self.data.field(idx)
 
                 # Delete the _arrays attribute so that it is recreated to
                 # point to the new data placed in the column objects above
                 del self.columns._arrays
-            except (TypeError, AttributeError):
-                # This shouldn't happen as long as self.columns._arrays
-                # is a lazyproperty
-                pass
         elif data is None:
             pass
         else:
@@ -377,23 +438,23 @@ class _TableBaseHDU(ExtensionHDU, _TableLikeHDU):
         # touch the data, so it's defined (in the case of reading from a
         # FITS file)
         self.data
-        return new_table(self.columns, header=self._header,
-                         tbtype=self.columns._tbtype)
+        return new_table(self.columns, header=self._header)
 
     def _prewriteto(self, checksum=False, inplace=False):
         if self._has_data:
-            self.data._scale_back()
+            self.data._scale_back(
+                update_heap_pointers=not self._manages_own_heap)
             # check TFIELDS and NAXIS2
             self._header['TFIELDS'] = len(self.data._coldefs)
             self._header['NAXIS2'] = self.data.shape[0]
 
             # calculate PCOUNT, for variable length tables
-            tbsize = self.header['NAXIS1'] * self.header['NAXIS2']
-            heapstart = self.header.get('THEAP', tbsize)
+            tbsize = self._header['NAXIS1'] * self._header['NAXIS2']
+            heapstart = self._header.get('THEAP', tbsize)
             self.data._gap = heapstart - tbsize
             pcount = self.data._heapsize + self.data._gap
             if pcount > 0:
-                self.header['PCOUNT'] = pcount
+                self._header['PCOUNT'] = pcount
 
             # update the other T****n keywords
             self._populate_table_keywords()
@@ -505,7 +566,7 @@ class TableHDU(_TableBaseHDU):
     def match_header(cls, header):
         card = header.cards[0]
         xtension = card.value
-        if isinstance(xtension, basestring):
+        if isinstance(xtension, string_types):
             xtension = xtension.rstrip()
         return card.keyword == 'XTENSION' and xtension == cls._extension
 
@@ -562,7 +623,7 @@ class TableHDU(_TableBaseHDU):
             # This is the case where the data has not been read from the file
             # yet.  We can handle that in a generic manner so we do it in the
             # base class.  The other possibility is that there is no data at
-            # all.  This can also be handled in a gereric manner.
+            # all.  This can also be handled in a generic manner.
             return super(TableHDU, self)._calculate_datasum(blocking)
 
     def _verify(self, option='warn'):
@@ -587,11 +648,12 @@ class BinTableHDU(_TableBaseHDU):
     _extension = 'BINTABLE'
     _ext_comment = 'binary table extension'
 
+
     @classmethod
     def match_header(cls, header):
         card = header.cards[0]
         xtension = card.value
-        if isinstance(xtension, basestring):
+        if isinstance(xtension, string_types):
             xtension = xtension.rstrip()
         return (card.keyword == 'XTENSION' and
                 xtension in (cls._extension, 'A3DTABLE'))
@@ -655,17 +717,28 @@ class BinTableHDU(_TableBaseHDU):
 
                 nbytes = self.data._gap
 
-                for idx in range(self.data._nfields):
-                    if not isinstance(self.data.columns._recformats[idx],
-                                      _FormatP):
-                        continue
+                if not self._manages_own_heap:
+                    # Write the heap data one column at a time, in the order
+                    # that the data pointers appear in the column (regardless
+                    # if that data pointer has a different, previous heap
+                    # offset listed)
+                    for idx in range(self.data._nfields):
+                        if not isinstance(self.data.columns._recformats[idx],
+                                          _FormatP):
+                            continue
 
-                    field = self.data.field(idx)
-                    for row in field:
-                        if len(row) > 0:
-                            nbytes += row.nbytes
-                            if not fileobj.simulateonly:
-                                fileobj.writearray(row)
+                        field = self.data.field(idx)
+                        for row in field:
+                            if len(row) > 0:
+                                nbytes += row.nbytes
+                                if not fileobj.simulateonly:
+                                    fileobj.writearray(row)
+                else:
+                    heap_data = self.data._get_heap_data()
+                    if len(heap_data) > 0:
+                        nbytes += len(heap_data)
+                        if not fileobj.simulateonly:
+                            fileobj.writearray(heap_data)
 
                 self.data._heapsize = nbytes - self.data._gap
                 size += nbytes
@@ -804,7 +877,7 @@ class BinTableHDU(_TableBaseHDU):
         files = [datafile, cdfile, hfile]
 
         for f in files:
-            if isinstance(f, basestring):
+            if isinstance(f, string_types):
                 if os.path.exists(f) and os.path.getsize(f) != 0:
                     if clobber:
                         warnings.warn("Overwriting existing file '%s'." % f,
@@ -934,7 +1007,7 @@ class BinTableHDU(_TableBaseHDU):
 
         close_file = False
 
-        if isinstance(fileobj, basestring):
+        if isinstance(fileobj, string_types):
             fileobj = open(fileobj, 'w')
             close_file = True
 
@@ -959,8 +1032,9 @@ class BinTableHDU(_TableBaseHDU):
 
             # Process each column of the row.
             for column in self.columns:
-                vla_format = None   # format of data in a variable length array
-                                    # where None means it is not a VLA
+                # format of data in a variable length array
+                # where None means it is not a VLA:
+                vla_format = None
                 format = _convert_format(column.format)
 
                 if isinstance(format, _FormatP):
@@ -1003,7 +1077,7 @@ class BinTableHDU(_TableBaseHDU):
 
         close_file = False
 
-        if isinstance(fileobj, basestring):
+        if isinstance(fileobj, string_types):
             fileobj = open(fileobj, 'w')
             close_file = True
 
@@ -1028,7 +1102,7 @@ class BinTableHDU(_TableBaseHDU):
 
         close_file = False
 
-        if isinstance(fileobj, basestring):
+        if isinstance(fileobj, string_types):
             fileobj = open(fileobj, 'r')
             close_file = True
 
@@ -1094,8 +1168,8 @@ class BinTableHDU(_TableBaseHDU):
         # can convert from this format to an actual FITS file on disk without
         # needing enough physical memory to hold the entire thing at once;
         # new_table() could use a similar feature.
-        hdu = new_table(np.recarray(shape=1, dtype=dtype), nrows=nrows,
-                        fill=True)
+        hdu = BinTableHDU.from_columns(np.recarray(shape=1, dtype=dtype),
+                                       nrows=nrows, fill=True)
         data = hdu.data
         for idx, length in enumerate(vla_lengths):
             if length is not None:
@@ -1168,7 +1242,7 @@ class BinTableHDU(_TableBaseHDU):
 
         close_file = False
 
-        if isinstance(fileobj, basestring):
+        if isinstance(fileobj, string_types):
             fileobj = open(fileobj, 'r')
             close_file = True
 
@@ -1193,12 +1267,10 @@ class BinTableHDU(_TableBaseHDU):
         return ColDefs(columns)
 
 
-@deprecated('0.3',
-            alternative=':meth:`FITS_rec.from_columns` to create a new '
-                        ':class:`FITS_rec` data object from the input '
-                        'columns to pass into the constructor for '
-                        ':class:`BinTableHDU` or :class:`TableHDU`',
-            pending=True)
+@deprecated('0.4',
+            alternative=':meth:`BinTableHDU.from_columns` for new BINARY '
+                        'tables or :meth:`TableHDU.from_columns` for new '
+                        'ASCII tables')
 def new_table(input, header=None, nrows=0, fill=False, tbtype=BinTableHDU):
     """
     Create a new table from the input column definitions.
@@ -1235,7 +1307,7 @@ def new_table(input, header=None, nrows=0, fill=False, tbtype=BinTableHDU):
 
     # tbtype defaults to classes now, but in all prior version of PyFITS it was
     # a string, so we still support that use case as well
-    if not isinstance(tbtype, basestring):
+    if not isinstance(tbtype, string_types):
         cls = tbtype
         tbtype = cls.__name__
     else:
@@ -1245,25 +1317,12 @@ def new_table(input, header=None, nrows=0, fill=False, tbtype=BinTableHDU):
         # hope is to deprecate this function anyways so there's not much point
         # in trying to make it more "generic".
         if tbtype.lower() == 'tablehdu':
-            tbtype = 'TableHDU'
             cls = TableHDU
         elif tbtype.lower() == 'bintablehdu':
-            tbtype = 'BinTableHDU'
             cls = BinTableHDU
         else:
             raise ValueError("tbtype must be one of 'TableHDU' or "
                              "'BinTableHDU'")
 
-    if isinstance(input, FITS_rec):  # input is a FITS_rec
-        # Create a new ColDefs object from the input FITS_rec's ColDefs
-        # object and assign it to the ColDefs attribute of the new hdu.
-        columns = ColDefs(input._coldefs, tbtype)
-    else:  # input is a list of Columns or possibly a recarray
-        # Create a new ColDefs object from the input list of Columns and
-        # assign it to the ColDefs attribute of the new hdu.
-        columns = ColDefs(input, tbtype)
-
-    data = FITS_rec.from_columns(columns, nrows=nrows, fill=fill)
-
     # construct a table HDU of the requested type
-    return cls(header=header, data=data)
+    return cls.from_columns(input, header=header, nrows=nrows, fill=fill)
