@@ -22,7 +22,7 @@ __all__ = ['read_table_hdf5', 'write_table_hdf5']
 
 def _find_all_structured_arrays(handle):
     """
-    Find all sturctured arrays in an HDF5 file
+    Find all structured arrays in an HDF5 file
     """
     import h5py
     structured_arrays = []
@@ -51,7 +51,7 @@ def is_hdf5(origin, filepath, fileobj, *args, **kwargs):
     except ImportError:
         return False
     else:
-        return isinstance(args[0], (h5py.highlevel.File, h5py.highlevel.Group))
+        return isinstance(args[0], (h5py.highlevel.File, h5py.highlevel.Group, h5py.highlevel.Dataset))
 
 
 def read_table_hdf5(input, path=None):
@@ -179,6 +179,13 @@ def write_table_hdf5(table, output, path=None, compression=False,
     except ImportError:
         raise Exception("h5py is required to read and write HDF5 files")
 
+    # Tables with mixin columns are not supported
+    if table.has_mixin_columns:
+        mixin_names = [name for name, col in table.columns.items()
+                       if not isinstance(col, table.ColumnClass)]
+        raise ValueError('cannot write table with mixin column(s) {0} to HDF5'
+                         .format(mixin_names))
+
     if path is None:
         raise ValueError("table path should be set via the path= argument")
     elif path.endswith('/'):
@@ -235,10 +242,10 @@ def write_table_hdf5(table, output, path=None, compression=False,
     if compression:
         if compression is True:
             compression = 'gzip'
-        dset = output_group.create_dataset(name, data=table._data,
+        dset = output_group.create_dataset(name, data=table.as_array(),
                                            compression=compression)
     else:
-        dset = output_group.create_dataset(name, data=table._data)
+        dset = output_group.create_dataset(name, data=table.as_array())
 
     # Write the meta-data to the file
     for key in table.meta:
