@@ -1,3 +1,7 @@
+# -*- coding: utf-8 -*-
+
+# TEST_UNICODE_LITERALS
+
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """
 This is a set of regression tests for vo.
@@ -9,7 +13,6 @@ from ....extern.six.moves import xrange
 
 # STDLIB
 import difflib
-from distutils import version
 import io
 import os
 import shutil
@@ -323,16 +326,16 @@ class TestParse:
         assert issubclass(self.array['unicode_test'].dtype.type,
                           np.object_)
         assert_array_equal(self.array['unicode_test'],
-                           ["Ce\xe7i n'est pas un pipe",
-                            '\u0bb5\u0ba3\u0b95\u0bcd\u0b95\u0bae\u0bcd',
+                           ["Ceçi n'est pas un pipe",
+                            'வணக்கம்',
                             'XXXX', '', ''])
 
     def test_fixed_unicode_test(self):
         assert issubclass(self.array['fixed_unicode_test'].dtype.type,
                           np.unicode_)
         assert_array_equal(self.array['fixed_unicode_test'],
-                           ["Ce\xe7i n'est",
-                            '\u0bb5\u0ba3\u0b95\u0bcd\u0b95\u0bae\u0bcd',
+                           ["Ceçi n'est",
+                            'வணக்கம்',
                             '0123456789', '', ''])
 
     def test_unsignedByte(self):
@@ -721,6 +724,8 @@ def test_open_files():
         parse(filename, pedantic=False)
 
     for filename in get_pkg_data_filenames('data', '*.xml'):
+        if filename.endswith('custom_datatype.xml'):
+            continue
         yield test_file, filename
 
 
@@ -874,9 +879,9 @@ def test_fileobj():
         if sys.platform == 'win32':
             fd()
         else:
-            if sys.version_info[0] >= 3:
+            if six.PY3:
                 assert isinstance(fd, io.FileIO)
-            else:
+            elif six.PY2:
                 assert isinstance(fd, file)
 
 
@@ -982,3 +987,20 @@ def test_no_resource_check():
                 replace('\\n', '\n'))
 
     assert truth == output
+
+
+def test_instantiate_vowarning():
+    # This used to raise a deprecation exception on Python 2.6.
+    # See https://github.com/astropy/astroquery/pull/276
+    VOWarning(())
+
+
+def test_custom_datatype():
+    votable = parse(
+        get_pkg_data_filename('data/custom_datatype.xml'),
+        pedantic=False,
+        datatype_mapping={'bar': 'int'}
+    )
+
+    table = votable.get_first_table()
+    assert table.array.dtype['foo'] == np.int32
