@@ -13,14 +13,13 @@ from __future__ import (absolute_import, unicode_literals, division,
 from ...extern import six
 
 from numpy.testing.utils import assert_allclose
-from ...tests.helper import raises, pytest
+from ...tests.helper import raises, pytest, catch_warnings
 
 from ... import units as u
 from ...constants import si
 from .. import core
 from .. import format as u_format
 from ..utils import is_effectively_unity
-from ... import wcs
 
 
 def test_unit_grammar():
@@ -178,7 +177,7 @@ def test_roundtrip():
         assert_allclose(b.decompose().scale, unit.decompose().scale, rtol=1e-2)
 
     for key, val in u.__dict__.items():
-        if isinstance(val, core.Unit) and not isinstance(val, core.PrefixUnit):
+        if isinstance(val, core.UnitBase) and not isinstance(val, core.PrefixUnit):
             yield _test_roundtrip, val
 
 
@@ -195,7 +194,7 @@ def test_roundtrip_vo_unit():
 
     x = u_format.VOUnit()
     for key, val in x._units.items():
-        if isinstance(val, core.Unit) and not isinstance(val, core.PrefixUnit):
+        if isinstance(val, core.UnitBase) and not isinstance(val, core.PrefixUnit):
             yield _test_roundtrip_vo_unit, val, val in (u.mag, u.dB)
 
 
@@ -206,7 +205,7 @@ def test_roundtrip_fits():
         assert_allclose(a.decompose().scale, unit.decompose().scale, rtol=1e-2)
 
     for key, val in u_format.Fits()._units.items():
-        if isinstance(val, core.Unit) and not isinstance(val, core.PrefixUnit):
+        if isinstance(val, core.UnitBase) and not isinstance(val, core.PrefixUnit):
             yield _test_roundtrip_fits, val
 
 
@@ -222,7 +221,7 @@ def test_roundtrip_cds():
 
     x = u_format.CDS()
     for key, val in x._units.items():
-        if isinstance(val, core.Unit) and not isinstance(val, core.PrefixUnit):
+        if isinstance(val, core.UnitBase) and not isinstance(val, core.PrefixUnit):
             yield _test_roundtrip_cds, val
 
 
@@ -238,7 +237,7 @@ def test_roundtrip_ogip():
 
     x = u_format.OGIP()
     for key, val in x._units.items():
-        if isinstance(val, core.Unit) and not isinstance(val, core.PrefixUnit):
+        if isinstance(val, core.UnitBase) and not isinstance(val, core.PrefixUnit):
             yield _test_roundtrip_ogip, val
 
 
@@ -258,10 +257,22 @@ def test_latex():
     fluxunit = u.erg / (u.cm ** 2 * u.s)
     assert fluxunit.to_string('latex') == r'$\mathrm{\frac{erg}{s\,cm^{2}}}$'
 
+
 def test_new_style_latex():
     fluxunit = u.erg / (u.cm ** 2 * u.s)
     assert "{0:latex}".format(fluxunit) == r'$\mathrm{\frac{erg}{s\,cm^{2}}}$'
 
+
+def test_latex_scale():
+    latex = '$\\mathrm{2.1798721 \\times 10^{-18}\\,\\frac{m^{2}\\,kg}{s^{2}}}$'
+    assert u.Ry.decompose().to_string('latex') == latex
+
+
+def test_latex_inline_scale():
+    latex_inline = '$\\mathrm{2.1798721 \\times 10^{-18}\\,m^{2}\\,kg\\,s^{-2}}$'
+    assert u.Ry.decompose().to_string('latex_inline') == latex_inline
+
+
 def test_format_styles():
     fluxunit = u.erg / (u.cm ** 2 * u.s)
     def _test_format_styles(format_spec, s):
@@ -272,12 +283,14 @@ def test_format_styles():
         ('s', 'erg / (cm2 s)'),
         ('console', '  erg  \n ------\n s cm^2'),
         ('latex', '$\\mathrm{\\frac{erg}{s\\,cm^{2}}}$'),
+        ('latex_inline', '$\\mathrm{erg\\,s^{-1}\\,cm^{-2}}$'),
         ('>20s','       erg / (cm2 s)'),
     ]
 
     for format_, s in format_s_pairs:
         yield _test_format_styles, format_, s
 
+
 def test_flatten_to_known():
     myunit = u.def_unit("FOOBAR_One", u.erg / u.Hz)
     assert myunit.to_string('fits') == 'erg Hz-1'
@@ -352,3 +365,78 @@ def test_scaled_dimensionless():
     with pytest.raises(ValueError):
         u.Unit(0.1).to_string('vounit')
 
+
+def test_deprecated_did_you_mean_units():
+    try:
+        u.Unit('ANGSTROM', format='fits')
+    except ValueError as e:
+        assert 'angstrom (deprecated)' in six.text_type(e)
+        assert 'Angstrom (deprecated)' in six.text_type(e)
+        assert 'nm (with data multiplied by 0.1)' in six.text_type(e)
+
+    with catch_warnings() as w:
+        u.Unit('Angstrom', format='fits')
+    assert len(w) == 1
+    assert 'nm (with data multiplied by 0.1)' in six.text_type(w[0].message)
+
+    try:
+        u.Unit('crab', format='ogip')
+    except ValueError as e:
+        assert 'Crab (deprecated)' in six.text_type(e)
+        assert 'mCrab (deprecated)' in six.text_type(e)
+
+    try:
+        u.Unit('ANGSTROM', format='vounit')
+    except ValueError as e:
+        assert 'angstrom (deprecated)' in six.text_type(e)
+        assert '0.1nm' in six.text_type(e)
+        assert six.text_type(e).count('0.1nm') == 1
+
+    with catch_warnings() as w:
+        u.Unit('angstrom', format='vounit')
+    assert len(w) == 1
+    assert '0.1nm' in six.text_type(w[0].message)
+
+
+def test_vounit_binary_prefix():
+    u.Unit('KiB', format='vounit') == u.Unit('1024 B')
+    u.Unit('Kibyte', format='vounit') == u.Unit('1024 B')
+    u.Unit('Kibit', format='vounit') == u.Unit('1024 B')
+    with catch_warnings() as w:
+        u.Unit('kibibyte', format='vounit')
+    assert len(w) == 1
+
+
+def test_vounit_unknown():
+    assert u.Unit('unknown', format='vounit') is None
+    assert u.Unit('UNKNOWN', format='vounit') is None
+    assert u.Unit('', format='vounit') is u.dimensionless_unscaled
+
+
+def test_vounit_details():
+    assert u.Unit('Pa', format='vounit') is u.Pascal
+
+    # The da- prefix is not allowed, and the d- prefix is discouraged
+    assert u.dam.to_string('vounit') == '10m'
+    assert u.Unit('dam dag').to_string('vounit') == '100g m'
+
+
+def test_vounit_custom():
+    x = u.Unit("'foo' m", format='vounit')
+    x_vounit = x.to_string('vounit')
+    assert x_vounit == "'foo' m"
+    x_string = x.to_string()
+    assert x_string == "foo m"
+
+    x = u.Unit("m'foo' m", format='vounit')
+    assert x.bases[1]._represents.scale == 0.001
+    x_vounit = x.to_string('vounit')
+    assert x_vounit == "m m'foo'"
+    x_string = x.to_string()
+    assert x_string == 'm mfoo'
+
+
+def test_vounit_implicit_custom():
+    x = u.Unit("furlong/week", format="vounit")
+    assert x.bases[0]._represents.scale == 1e-15
+    assert x.bases[0]._represents.bases[0].name == 'urlong'
